import React, { useEffect, useState } from 'react';
import { ethers, BigNumber } from 'ethers';
import { useAccount, useContractReads, useContractRead, usePrepareContractWrite, useContractWrite, useWaitForTransaction, erc721ABI } from 'wagmi';
import { NotificationContainer, NotificationManager } from 'react-notifications';
import { StatsPanel, TrollboxPanel } from './Panels';

import MainABI from '../abi/main.json';
import '../styles/hero.css';
import 'react-notifications/lib/notifications.css';
import Unaboomer from '../img/unaboomer.png';
import Mailbomb from '../img/mailbomb.png';
import Explosion from '../img/explosion.png';
import Arrow from '../img/arrow_right.png';
import LZA from '../img/lza_menace.png';
import Carty from '../img/cartyisme.png';
import Loading from '../img/loading.svg';
import Splash from '../img/splash_desktop_slim.png';


export function Hero(props) {
    const contractAddress = props.contractAddress;
    const { isConnected, address } = useAccount();
    const [throttled, setThrottled] = useState('');

    const [tokensMinted, setTokensMinted] = useState(() => {
        const saved = localStorage.getItem('tokensMinted-v3-' + address);
        return saved || JSON.stringify([])
    });

    useEffect(() => {
        localStorage.setItem('tokensMinted-v3-' + address, tokensMinted);
    }, [tokensMinted]);

    const [options, setOptions] = useState({ 
        unaboomerAmount: 1,
        bombAmount: 1,
        sendBombAmount: 1,
        unaboomerPreviewAmount: 1,
        bombPreviewAmount: 1,
        sendBombPreviewAmount: 1,
        unaboomerPrice: ethers.utils.parseEther('.005'),
        bombPrice: ethers.utils.parseEther('.0025'),
        unaboomersMinted: 0,
        unaboomerBalance: 0,
        bombBalance: 0,
        unaboomersRadicalized: 0,
        bombsAssembled: 0,
        bombsExploded: 0,
        unaboomersKilled: 0,
        leaderboardPointer: 0,
        leaderAddress: '',
        leaderKillCount: 0,
        unaboomerMaxSupply: 0,
        unaboomerMaxSurvivorCount: 0,
        unaboomerMaxMintPerWallet: 0,
        balancesFetched: false,
        unaboomersByAddress: []
    });
    function handleStateChange(obj) {
        setOptions(preState => ({...preState , ...obj}));
    }
    function throttleStateChange(obj) {
        const og = throttled;
        let t = setTimeout(function() {
            setOptions(preState => ({...preState , ...obj}));
        }, 600);
        setThrottled(t);
        clearTimeout(og);
    }
    const defOpt = {
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected
    }
    // Fetch this stuff and cache aggressively since it doesn't change
    useContractReads({
        contracts: [
            {...defOpt, functionName: 'unaboomerPrice'},
            {...defOpt, functionName: 'bombPrice'},
            {...defOpt, functionName: 'unaboomerMaxSupply'},
            {...defOpt, functionName: 'unaboomerMaxSurvivorCount'},
            {...defOpt, functionName: 'unaboomerMaxMintPerWallet'}
        ],
        watch: true,
        enabled: isConnected,
        cacheTime: 30_000,
        onSuccess(data) {
            handleStateChange({
                unaboomerPrice: data[0].toString(),
                bombPrice: data[1].toString(),
                unaboomerMaxSupply: Number(data[2]),
                unaboomerMaxSurvivorCount: Number(data[3]),
                unaboomerMaxMintPerWallet: Number(data[4])
            });
        }
    });
    // Fetch this stuff more frequently as it updates constantly
    useContractReads({
        contracts: [
            {...defOpt, functionName: 'unaboomersMinted', args: [address]},
            {...defOpt, functionName: 'bombBalance', args: [address]},
            {...defOpt, functionName: 'unaboomersRadicalized'},
            {...defOpt, functionName: 'bombsAssembled'},
            {...defOpt, functionName: 'bombsExploded'},
            {...defOpt, functionName: 'unaboomersKilled'},
            {...defOpt, functionName: 'leaderboardPointer'},
            {...defOpt, functionName: 'unaboomerBalance', args: [address]}
        ],
        watch: true,
        enabled: isConnected,
        cacheTime: 6_000,
        onSuccess(data) {
            handleStateChange({
                unaboomersMinted: Number(data[0]),
                bombBalance: Number(data[1]),
                unaboomersRadicalized: Number(data[2]),
                bombsAssembled: Number(data[3]),
                bombsExploded: Number(data[4]),
                unaboomersKilled: Number(data[5]),
                leaderboardPointer: Number(data[6]),
                balancesFetched: true,
                unaboomerBalance: Number(data[7])
            });
        }
    });
    useContractRead({
        address: contractAddress,
        abi: MainABI,
        enabled: options.leaderboardPointer > 0,
        functionName: 'leaderboard',
        args: [options.leaderboardPointer],
        watch: true,
        cacheTime: 8_000,
        onSuccess: async (data) => {
            handleStateChange({
                leaderAddress: data,
            });
        }
    });
    useContractRead({
        address: contractAddress,
        abi: MainABI,
        enabled: options.leaderAddress.length > 0,
        functionName: 'killCount',
        args: [options.leaderAddress],
        watch: true,
        cacheTime: 8_000,
        onSuccess(data) {
            handleStateChange({
                leaderKillCount: Number(data),
            });
        }
    });
    const radicalizeBoomersPrepare = usePrepareContractWrite({
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected && options.unaboomersMinted < options.unaboomerMaxMintPerWallet && options.unaboomerAmount > 0 && options.balancesFetched,
        functionName: 'radicalizeBoomers',
        args: [options.unaboomerAmount],
        overrides: {
            from: address,
            value: BigNumber.from((options.unaboomerPrice * options.unaboomerAmount).toString())
        }
    });
    const radicalizeBoomersWrite = useContractWrite({
        ...radicalizeBoomersPrepare.config,
        onError(data) {
            if (data.message.startsWith('user rejected transaction')) NotificationManager.info(`tx cancelled`, 'ok', 4000);
            handleStateChange({
                unaboomerAmount: 0
            })
        },
        onSuccess(data) {
            handleStateChange({
                unaboomerAmount: 0
            })
        }
    });
    const assembleBombsPrepare = usePrepareContractWrite({
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected && options.bombAmount > 0,
        functionName: 'assembleBombs',
        args: [options.bombAmount],
        overrides: {
            from: address,
            value: BigNumber.from((options.bombPrice * options.bombAmount).toString())
        }
    });
    const assembleBombsWrite = useContractWrite({
        ...assembleBombsPrepare.config,
        onError(data) {
            if (data.message.startsWith('user rejected transaction')) NotificationManager.info(`tx cancelled`, 'ok', 4000);
        },
        onSuccess(data) {
            NotificationManager.success(`Bombs acquired. Let's kill some Boomers.`, '', 3000);
            assembleBombsWrite.reset();
        }
    });
    const sendBombsPrepare = usePrepareContractWrite({
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected && options.bombBalance > 0,
        functionName: 'sendBombs',
        cacheTime: 0,
        staleTime: 400,
        args: [options.sendBombAmount]
    });
    const sendBombsWrite = useContractWrite({
        ...sendBombsPrepare.config,
        onError(data) {
            if (data.message.startsWith('user rejected transaction')) NotificationManager.info(`tx cancelled`, 'ok', 4000);
        }
    });
    useWaitForTransaction({
        hash: radicalizeBoomersWrite.data?.hash,
        enabled: radicalizeBoomersWrite.status === 'success',
        onSuccess(data) {
            let tm = JSON.parse(tokensMinted);
            const iface = new ethers.utils.Interface(erc721ABI);
            data.logs.filter(log => 
                log.topics[0] === '0xddf252ad1be2c89b69c2b068fc378daa952ba7f163c4a11628f55a4df523b3ef'
            ).map((log) => {
                const res = iface.decodeEventLog("Transfer", log.data, log.topics);
                NotificationManager.success(`You radicalized Unaboomer ${res.tokenId.toString()}`, '', 4000);
                tm.push(Number(res.tokenId));
            });
            setTokensMinted(JSON.stringify(tm));
            radicalizeBoomersWrite.reset();
        }
    })
    useWaitForTransaction({
        hash: sendBombsWrite.data?.hash,
        enabled: sendBombsWrite.status === 'success',
        onSuccess(data) {
            const iface = new ethers.utils.Interface(MainABI);
            data.logs.filter(log => 
                log.topics[0] === '0x38c7e2e8001cefb27535bd6bcc6363c6d6a6a83dbb77f092b78956334596f9ed'
            ).map((log) => {
                const res = iface.decodeEventLog("SentBomb", log.data, log.topics);
                if (res.hit) {
                    if (res.owned) {
                        NotificationManager.error(`Your bomb exploded during assembly and killed your Unaboomer ${res.tokenId.toString()}`, '', 5000);
                    } else {
                        NotificationManager.success(`Your bomb killed Unaboomer ${res.tokenId.toString()}`, '', 5000);
                    }
                } else {
                    NotificationManager.warning(`Your bomb for Unaboomer ${res.tokenId.toString()} was a dud`, '', 5000);
                }
            });
            sendBombsWrite.reset();
        }
    });

    return (
        <div id='hero'>
            <NotificationContainer />
            <h1 id='heroTitle'>the web3 revolution & its consequences have been a disaster for the human race.</h1>
            <div id='heroFlex'>
                <div className='heroFlexItem'>
                    <img src={Unaboomer} width='280px' alt='Pixelated Unaboomer profile picture - a hooded man with dark sunglasses.' />
                    <p>
                        Technology is leaving the BOOMER generation behind. 
                        Now they're on a war path to disrupt the metaverse.
                        <br /><br />
                        5,000 BOOMERS are ready to radicalize.
                        <br /><br />
                        Only 1,000 BOOMERS will survive.
                        <br /><br />
                    </p>
                    {options.unaboomersMinted > 0 && (
                        <p>
                            <br />
                            You have radicalized {options.unaboomersMinted} Unaboomers.
                        </p>
                    )}
                    <input type='number' step='1' defaultValue={1} min={1} max={options.unaboomerMaxMintPerWallet - options.unaboomersMinted} onChange={(v) => {
                        throttleStateChange({unaboomerAmount: v.target.value});
                        radicalizeBoomersWrite.reset()
                    }}></input>
                    <button className='doThing' disabled={radicalizeBoomersPrepare.status == 'error' || !isConnected} onClick={() => radicalizeBoomersWrite.write?.()}>
                        {radicalizeBoomersWrite.isLoading && <>minting {options.unaboomerAmount}</>} 
                        {radicalizeBoomersWrite.isIdle && <>Mint {options.unaboomerAmount == 1 ? 'a Boomer': <>{options.unaboomerAmount} Boomers</>}</>}
                        {radicalizeBoomersWrite.isSuccess && <img src={Loading} width="25px"></img>}
                        {radicalizeBoomersWrite.isError && <>error</>}
                    </button>
                    <p>({ethers.utils.formatEther((options.unaboomerPrice * options.unaboomerAmount).toString())} ETH)</p>
                </div>
                <div className='heroFlexArrow'>
                    <img src={Arrow} width='100px' alt='Arrow pointing to the next step.' />
                </div>
                <div className='heroFlexItem'>
                    <img src={Mailbomb} width='280px' alt='Pixelated mail bomb image - handle with caution.' />
                    <p>
                        Don't let the BOOMERS win! 
                        <br /><br />
                        Assemble mail bombs and blow 'em to smithereens. 
                        BOOMERS will be airdropped mail bombs upon mint.
                        <br /><br />
                        Burned mail bombs destroy a BOOMER forever.
                        <br /><br />
                        (Warning: Some bombs are duds!)
                    </p>
                    {options.bombBalance > 0 && (
                        <p>
                            <br />
                            You have assembled {options.bombBalance} mail bombs.
                        </p>
                    )}
                    <input type='number' step='1' defaultValue={1} min={1} onChange={(v) => {
                        throttleStateChange({bombAmount: v.target.value});
                        assembleBombsWrite.reset()
                    }}></input>
                    <button className='doThing' disabled={assembleBombsPrepare.status == 'error' || !isConnected} onClick={() => assembleBombsWrite.write?.()}>
                        {assembleBombsWrite.isLoading && <>minting {options.bombAmount}</>} 
                        {assembleBombsWrite.isIdle && <>Mint {options.bombAmount == 1 ? 'a mail bomb': <>{options.bombAmount} Mail bombs</>}</>}
                        {assembleBombsWrite.isSuccess && <img src={Loading} width="25px"></img>}
                        {assembleBombsWrite.isError && <>error</>}
                    </button>
                    <p>({ethers.utils.formatEther((options.bombPrice * options.bombAmount).toString())} ETH)</p>
                </div>
                <div className='heroFlexArrow'>
                    <img src={Arrow} width='100px' alt='Arrow pointing to the next step.' />
                </div>
                <div className='heroFlexItem'>
                    <img src={Explosion} width='280px' alt='Pixelated explosion image.' />
                    <p>
                        Let the mayhem begin! Ship a mail bomb to destroy a random BOOMER in the collection.
                        <br /><br />
                        Destroyed BOOMERS explode, lose their traits, and burn.
                        <br /><br />
                        The game ends when 1,000 BOOMERS are left standing. Shipments begin at a time of our choosing.
                    </p>
                    <input type='number' step='1' defaultValue={1} min={1} max={options.bombBalance} onChange={(v) => {
                        throttleStateChange({sendBombAmount: v.target.value});
                        sendBombsWrite.reset()
                    }}></input>
                    <button className='doThing' disabled={sendBombsPrepare.status == 'error' || !isConnected} onClick={() => sendBombsWrite.write?.()}>
                        {sendBombsWrite.isLoading && <>mailing {options.sendBombAmount}</>} 
                        {sendBombsWrite.isIdle && <>Mail {options.sendBombAmount == 1 ? 'a bomb': <>{options.sendBombAmount} Mail bombs</>}</>}
                        {sendBombsWrite.isSuccess && <img src={Loading} width="25px"></img>}
                        {sendBombsWrite.isError && <>error</>}
                    </button>
                    <p>(tx fee)</p>
                </div>
            </div>
            {isConnected && options.unaboomersRadicalized > 0 && (
                <>
                <div className="progress">
                    <p>{options.unaboomersRadicalized} / {options.unaboomerMaxSupply} minted</p>
                    <div className="progress-value" style={{ 
                        width: (options.unaboomersRadicalized / options.unaboomerMaxSupply * 100) + '%'
                    }}>
                    </div>
                </div>
                <div className="progress">
                    <p>{options.unaboomersKilled} / {options.unaboomerMaxSupply - options.unaboomerMaxSurvivorCount} killed</p>
                    <div className="progress-value" style={{ 
                        width: (options.unaboomersKilled / (options.unaboomerMaxSupply - options.unaboomerMaxSurvivorCount) * 100) + '%',
                        backgroundColor: 'red'
                    }}>
                    </div>
                </div>
                </>
            )}
            
            {isConnected && JSON.parse(tokensMinted).length > 0 && false && (
                <div id="userInventory">
                    <h1>You Minted</h1>
                    <div className="ownedTokens">
                        {JSON.parse(tokensMinted).filter((item, index) => JSON.parse(tokensMinted).indexOf(item) === index).map((id, idx) => (
                                <div className="ownedToken" key={idx}>
                                    <a href={'https://unaboomer.gg/images/' + String(id).padStart(4, '0') + '.png'} target='_blank'>
                                        <img src={'https://unaboomer.gg/images/' + String(id).padStart(4, '0') + '.png'} width="80px" />
                                    </a>
                                </div>
                            )
                        )}
                    </div>
                </div>
            ) || (
                <div id="boomerPreviewBanner">
                    <img src={Splash} width="100%" />
                </div>
            )}
            
            <div id="mintInfo" className="section">
                <h1>WTF!?</h1>
                <p>
                    UNABOOMER is a deflationary Player vs. Player PFP NFT consisting of 5,000 pixel BOOMERS with 190 unique traits.
                </p>
                <p>
                    Burn a mail bomb to destroy a random BOOMER. Destroyed BOOMERS explode, lose their traits, and burn. But be careful! Your BOOMER can blow itself up in the process. 
                </p>
                <p>
                    Mail bomb assembly is open to all. The more BOOMERS destroyed, the more likely your bomb is a dud. Shipments begin at a time of our choosing. 
                </p>
                <p>
                    5,000 BOOMERS are ready to radicalize. Only 1,000 BOOMERS will survive.
                </p>
                <p>
                    No allowlist, no creator fee, no roadmap, cc0.
                </p>
            </div>
            <div id="teamInfo" className="section">
                <h1>Team!</h1>
                <div id="teamMembers">
                    <div className='teamMember'>
                        <img src={Carty} width='180px' alt='cartyisme Renga PFP' />
                        <a href='https://twitter.com/cartyisme' target='_blank' className=''>@cartyisme</a>
                        <p>Artist, idiot, shillosopher, and Art101 progenitor.</p>
                    </div>
                    <div className='teamMember'>
                        <img src={LZA} width='180px' alt='lza_menace Milady PFP' />
                        <a href='https://twitter.com/lza_menace' target='_blank' className=''>@lza_menace</a>
                        <p>Coder, loser, Monero fan, and meme consooomer.</p>
                    </div>
                </div>
            </div>

            <div id="externalLinks">
                <a href="https://twitter.com/unaboomernft" target={"_blank"}>TWITTER</a>
                \\
                <a href="https://opensea.io/collection/unaboomer" target={"_blank"}>BOOMER OPENSEA</a>
                \\
                <a href="https://opensea.io/collection/unaboomer-bomb" target={"_blank"}>BOMB OPENSEA</a>
                <br />
                <br />
                <br />
            </div>

            {isConnected && address && (
                <>
                    <StatsPanel 
                        leaderKillCount={options.leaderKillCount}
                        leaderAddress={options.leaderAddress}
                        unaboomersRadicalized={options.unaboomersRadicalized}
                        unaboomersKilled={options.unaboomersKilled}
                        bombsAssembled={options.bombsAssembled}
                        bombsExploded={options.bombsExploded}
                    />
                    <TrollboxPanel 
                        ws={props.ws}
                        address={address} 
                    />
                </>
            )}
        </div>
    )
}
