import React, { useState, useEffect } from 'react';
import { ethers, BigNumber } from 'ethers';
import { useAccount, useContractReads, useContractRead, usePrepareContractWrite, useContractWrite, useWaitForTransaction, erc721ABI } from 'wagmi';
import { Parallax } from 'react-parallax';
import { NotificationContainer, NotificationManager } from 'react-notifications';
import { AwesomeButton } from 'react-awesome-button';
import useWebSocket, { ReadyState } from 'react-use-websocket';
import Typewriter from 'react-ts-typewriter';
import Slider from 'rc-slider';

import LZA from '../img/lza_menace.png';
import Carty from '../img/cartyisme.png';
import BoomerBW from '../img/unaboomer_bw.png';
import Boomer from '../img/unaboomer.png';
import Newspapers from '../img/newspapers.png';
import Bomb from '../img/bomb.png';
import Explosion from '../img/explosion.png';
import SeizedItems from '../img/seizeditems.png';
import Map from '../img/map.png';
import MainABI from '../abi/main.json';

import 'react-notifications/lib/notifications.css';
import 'rc-slider/assets/index.css';
import '../styles/sections.css';


export class Section1 extends React.Component {
    constructor(props) {
        super(props);
        this.state = {title: '', showImage: false};
        setInterval(() => {
            this.setState({
                title: 'The Web3 Revolution and its consequences have been a disaster for the human race.',
                showImage: true
            });
        }, 1200);
    }

    render() {
        return (
            <section id="section1" className="section">
                <h1 id="heroText">
                    <Typewriter text={this.state.title} cursor={true} random={20} />
                </h1>
                <img src={BoomerBW} id="heroImage" className={!this.state.showImage ? "hidden": "" } alt="Pixelated police sketch of the Unabomber suspect." />
                <div className={!this.state.showImage ? "hidden fadeIn": "fadeIn" }>
                    <svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M7.75735 5.63605L6.34314 7.05026L12 12.7071L17.6569 7.05029L16.2427 5.63608L12 9.87872L7.75735 5.63605Z" fill="currentColor" />
                        <path d="M6.34314 12.7071L7.75735 11.2929L12 15.5356L16.2427 11.2929L17.6569 12.7071L12 18.364L6.34314 12.7071Z" fill="currentColor" />
                    </svg>
                </div>
                
            </section>
        )
    }
}

export function Section2() {
    return (
        <div className="fadeIn">
            <Parallax blur={{ min: 0, max: 3 }} bgImage={Newspapers} bgImageAlt="Newspapers containing the Unabomber's manifesto." strength={300}>
                <section id="section2" className="section"></section>
            </Parallax>
            <Parallax blur={{ min: 0, max: 3 }} bgImage={SeizedItems} bgImageAlt="The Unabomber's tools and typewriter seized from his Montana cabin." strength={-80}>
                <section className="section"></section>
            </Parallax>
        </div>
    )
}

class GenerateBombOverlay extends React.Component {
    constructor(props) {
        super(props);
        let bombCoords = this.generateCoords(this.ifOver(props.amount));
        this.state = {coords: bombCoords, color: props.color};
    }

    ifOver(amt) {
        if (amt > 200) {
            return 200;
        } else {
            return amt;
        }
    }

    generateCoords(amt) {
        let bombCoords = [];
        for (let i = 0; i < amt; i++) {
            let randX = Math.floor(Math.random() * 99);
            let randY = Math.floor(Math.random() * 99);
            let o = {
                x: `${randX}%`,
                y: `${randY}%`
            }
            bombCoords.push(o);
        }
        return bombCoords;
    }

    componentDidUpdate(prevProps) {
        if (this.props.amount !== prevProps.amount) {
            let bombCoords = this.generateCoords(this.ifOver(this.props.amount));
            this.setState({
                coords: bombCoords
            });
        }
    }

    render() {
        return this.state.coords.map((obj, idx) => <div key={idx} className={this.props.type} style={{top: obj.y, left: obj.x, backgroundColor: this.state.color}} />)
    }
}

class GameMap extends React.Component {
    constructor(props) {
        super(props);
        this.state = {showStats: false, showMap: false}
    }

    shorten(s) {
        return s.slice(0, 6) + '...' + s.slice(-4)
    }

    render() {
        return (
            <>
                <div id="gameMap">
                    {this.state.showMap && (
                        <>
                            <h1 id="gameMapTitle">Metaverse Map</h1>
                            <p id="gameMapText">
                                {this.props.unaboomersRadicalized} / {this.props.unaboomerMaxSupply} Unaboomers radicalized and ready to terrorize the metaverse. {this.props.unaboomersKilled} killed.<br />
                                This round will end and the project will fully start when {this.props.unaboomerMaxSurvivorCount} or less Unaboomers remain standing.<br />
                            </p>
                            <div className="map">
                                <img src={Map} />
                                <GenerateBombOverlay amount={this.props.bombsAssembled - this.props.bombsExploded} color={'#E3B505'} type={'dots'} />
                                <GenerateBombOverlay amount={this.props.bombsExploded} color={'#D01500'} type={'dots'} />
                                <GenerateBombOverlay amount={this.props.bombsExploded - this.props.unaboomersKilled} color={'#0095FF'} type={'dots'} />
                            </div>
                        </>
                    )}
                    <button className="button" onClick={() => {
                        this.setState({showMap: !this.state.showMap})
                    }}>
                        {this.state.showMap && (<>hide map</>) || <>show map</>}
                    </button>
                    
                </div>
                <div id="statsbox">
                    {this.state.showStats && (
                        <div className="m2">
                            <img src={Boomer} width="50px" />
                            <p className="nopad">
                                Kill Leader:<br />
                                {this.props.leaderKillCount > 0 && (
                                    <a href={"https://etherscan.io/address/" + this.props.leaderAddress} target="_blank">
                                        {this.shorten(this.props.leaderAddress)}
                                    </a>
                                ) || "?"}
                                
                            </p>
                            {this.props.leaderKillCount > 0 && (
                                <p className="nopad">
                                    {this.props.leaderKillCount} kills
                                </p>
                            )}
                            
                            <hr />
                            <p className="nopad">
                                Active Unaboomers: {this.props.unaboomersRadicalized - this.props.unaboomersKilled} <br />
                                Dead Unaboomers: {this.props.unaboomersKilled} <br />
                                <i className="dots" style={{backgroundColor: '#E3B505', offset: '-20px'}}></i>: Active Bombs: {this.props.bombsAssembled - this.props.bombsExploded} <br />
                                <i className="dots" style={{backgroundColor: '#D01500'}}></i>: Exploded Bombs: {this.props.bombsExploded} <br />
                                <i className="dots" style={{backgroundColor: '#0095FF'}}></i>: Dud Bombs: {this.props.bombsExploded - this.props.unaboomersKilled} <br />
                            </p>
                        </div>
                    )}
                    <button className="button" onClick={() => {
                        this.setState({showStats: !this.state.showStats})
                    }}>
                        {this.state.showStats && (<>hide stats</>) || <>show stats</>}
                    </button>
                </div>
            </>
        )
    }
}

function Section3(props) {
    const contractAddress = props.contractAddress;
    const [tokensMinted, setTokensMinted] = useState(() => {
        const saved = localStorage.getItem('tokensMinted0');
        return saved || JSON.stringify([])
    });

    useEffect(() => {
        localStorage.setItem('tokensMinted0', tokensMinted);
    }, [tokensMinted]);

    const [options, setOptions] = useState({ 
        unaboomerAmount: 1,
        bombAmount: 1,
        sendBombAmount: 1,
        unaboomerPreviewAmount: 1,
        bombPreviewAmount: 1,
        sendBombPreviewAmount: 1,
        unaboomerPrice: 0,
        bombPrice: ethers.utils.parseEther('.01'),
        unaboomersMinted: 0,
        unaboomerBalance: 0,
        bombBalance: 0,
        unaboomersRadicalized: 0,
        bombsAssembled: 0,
        bombsExploded: 0,
        unaboomersKilled: 0,
        leaderboardPointer: 0,
        leaderAddress: '',
        leaderKillCount: 0,
        unaboomerMaxSupply: 0,
        unaboomerMaxSurvivorCount: 0,
        unaboomerMaxMintPerWallet: 0,
        readWTF: false,
        balancesFetched: false,
        unaboomersByAddress: []
    });
    function handleStateChange(obj) {
        setOptions(preState => ({...preState , ...obj}));
    }
    const { isConnected, address } = useAccount();
    const defOpt = {
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected
    }
    // Fetch this stuff and cache aggressively since it doesn't change
    useContractReads({
        contracts: [
            {
                ...defOpt,
                functionName: 'unaboomerPrice'
            },
            {
                ...defOpt,
                functionName: 'bombPrice'
            },
            {
                ...defOpt,
                functionName: 'unaboomerMaxSupply'
            },
            {
                ...defOpt,
                functionName: 'unaboomerMaxSurvivorCount'
            },
            {
                ...defOpt, 
                functionName: 'unaboomerMaxMintPerWallet'
            }
        ],
        watch: true,
        cacheTime: 30_000,
        onSuccess(data) {
            handleStateChange({
                unaboomerPrice: data[0].toString(),
                bombPrice: data[1].toString(),
                unaboomerMaxSupply: Number(data[2]),
                unaboomerMaxSurvivorCount: Number(data[3]),
                unaboomerMaxMintPerWallet: Number(data[4])
            });
        }
    });
    // Fetch this stuff more frequently as it updates constantly
    useContractReads({
        contracts: [
            {
                ...defOpt,
                functionName: 'unaboomersMinted',
                args: [address]
            },
            {
                ...defOpt,
                functionName: 'bombBalance',
                args: [address]
            },
            {
                ...defOpt,
                functionName: 'unaboomersRadicalized'
            },
            {
                ...defOpt,
                functionName: 'bombsAssembled'
            },
            {
                ...defOpt,
                functionName: 'bombsExploded'
            },
            {
                ...defOpt,
                functionName: 'unaboomersKilled'
            },
            {
                ...defOpt,
                functionName: 'leaderboardPointer'
            },
            {
                ...defOpt,
                functionName: 'unaboomerBalance',
                args: [address]
            }
        ],
        watch: true,
        cacheTime: 6_000,
        onSuccess(data) {
            handleStateChange({
                unaboomersMinted: Number(data[0]),
                bombBalance: Number(data[1]),
                unaboomersRadicalized: Number(data[2]),
                bombsAssembled: Number(data[3]),
                bombsExploded: Number(data[4]),
                unaboomersKilled: Number(data[5]),
                leaderboardPointer: Number(data[6]),
                balancesFetched: true,
                unaboomerBalance: Number(data[7])
            });
        }
    });
    useContractRead({
        address: contractAddress,
        abi: MainABI,
        enabled: options.leaderboardPointer > 0,
        functionName: 'leaderboard',
        args: [options.leaderboardPointer],
        watch: true,
        cacheTime: 8_000,
        onSuccess: async (data) => {
            handleStateChange({
                leaderAddress: data,
            });
        }
    });
    useContractRead({
        address: contractAddress,
        abi: MainABI,
        enabled: options.leaderAddress.length > 0,
        functionName: 'killCount',
        args: [options.leaderAddress],
        watch: true,
        cacheTime: 8_000,
        onSuccess(data) {
            handleStateChange({
                leaderKillCount: Number(data),
            });
        }
    });
    const radicalizeBoomersPrepare = usePrepareContractWrite({
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected && options.unaboomersMinted < options.unaboomerMaxMintPerWallet && options.unaboomerAmount > 0 && options.balancesFetched,
        functionName: 'radicalizeBoomers',
        args: [options.unaboomerAmount],
        overrides: {
            from: address,
            value: BigNumber.from((options.unaboomerPrice * options.unaboomerAmount).toString())
        }
    });
    const radicalizeBoomersWrite = useContractWrite({
        ...radicalizeBoomersPrepare.config,
        onError(data) {
            if (data.message.startsWith('user rejected transaction')) NotificationManager.info(`tx cancelled`, 'ok', 4000);
        }
    });
    const assembleBombsPrepare = usePrepareContractWrite({
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected && options.bombAmount > 0,
        functionName: 'assembleBombs',
        args: [options.bombAmount],
        overrides: {
            from: address,
            value: BigNumber.from((options.bombPrice * options.bombAmount).toString())
        }
    });
    const assembleBombsWrite = useContractWrite({
        ...assembleBombsPrepare.config,
        onError(data) {
            if (data.message.startsWith('user rejected transaction')) NotificationManager.info(`tx cancelled`, 'ok', 4000);
        }
    });
    const sendBombsPrepare = usePrepareContractWrite({
        address: contractAddress,
        abi: MainABI,
        enabled: isConnected && options.bombBalance > 0,
        functionName: 'sendBombs',
        cacheTime: 0,
        staleTime: 400,
        args: [options.sendBombAmount]
    });
    const sendBombsWrite = useContractWrite({
        ...sendBombsPrepare.config,
        onError(data) {
            if (data.message.startsWith('user rejected transaction')) NotificationManager.info(`tx cancelled`, 'ok', 4000);
        }
    });
    useWaitForTransaction({
        hash: radicalizeBoomersWrite.data?.hash,
        enabled: radicalizeBoomersWrite.status === 'success',
        onSuccess(data) {
            let tm = JSON.parse(tokensMinted);
            const iface = new ethers.utils.Interface(erc721ABI);
            data.logs.filter(log => 
                log.topics[0] === '0xddf252ad1be2c89b69c2b068fc378daa952ba7f163c4a11628f55a4df523b3ef'
            ).map((log) => {
                const res = iface.decodeEventLog("Transfer", log.data, log.topics);
                NotificationManager.success(`You radicalized Unaboomer #${res.tokenId.toString()}`, '', 4000);
                tm.push(Number(res.tokenId));
            });
            setTokensMinted(JSON.stringify(tm));
            console.log(tm);
        }
    })
    useWaitForTransaction({
        hash: sendBombsWrite.data?.hash,
        enabled: sendBombsWrite.status === 'success',
        onSuccess(data) {
            const iface = new ethers.utils.Interface(MainABI);
            data.logs.filter(log => 
                log.topics[0] === '0x38c7e2e8001cefb27535bd6bcc6363c6d6a6a83dbb77f092b78956334596f9ed'
            ).map((log) => {
                const res = iface.decodeEventLog("SentBomb", log.data, log.topics);
                if (res.hit) {
                    if (res.owned) {
                        NotificationManager.error(`Your bomb exploded during assembly and killed your Unaboomer ${res.tokenId.toString()}`, '', 8000);
                    } else {
                        NotificationManager.success(`Your bomb killed Unaboomer ${res.tokenId.toString()}`, '', 8000);
                    }
                } else {
                    NotificationManager.warning(`Your bomb for Unaboomer ${res.tokenId.toString()} was a dud`, '', 8000);
                }
            });
        }
    });
    return (
        <section className="section" id="mintSection">
            {!isConnected && (
                <div id="heroText">
                    <h1>connect your web3 wallet</h1>
                </div>
            )}

            {isConnected && options.unaboomersKilled >= options.unaboomerMaxSupply - options.unaboomerMaxSurvivorCount && options.unaboomerMaxSupply > 0 && (
                <div id="heroText">
                    <h4 style={{margin: 0}}>the culling of weak warriors has completed</h4>
                    <p>
                        {options.unaboomersRadicalized - options.unaboomersKilled} Unaboomers remain standing.<br /><br />
                        These warriors will advance to the next round of the game where they will work together 
                        to continue to terrorize the metaverse in order to lead a revolution.
                    </p>
                    <p style={{fontSize: '.8em', color: '#FF3E41'}}>We will not rest until our objective is complete.</p>
                </div>
            )}

            {isConnected && options.unaboomersKilled < options.unaboomerMaxSupply - options.unaboomerMaxSurvivorCount &&
            <>
                {!options.readWTF && (
                    <>
                        <h1 id="mintTitle">
                            Web3 Community and Its Future
                        </h1>
                        <p id="mintText">
                            The degens are taking us all on an utterly reckless ride into the unknown.<br /><br />
                            Many people understand something of what Web3 progress is doing to us yet take a passive attitude toward degeneracy because they think it is inevitable.<br /><br />
                            But we don’t think it is inevitable. We think it can be stopped. <br /><br />
                            The two main tasks for the present are to <span className="strong">promote social stress and instability in the Web3 community</span> and <span className="strong">to develop and propagate an ideology that opposes degeneracy and the corporate venture capital system</span>. When the system becomes sufficiently stressed and unstable, a revolution against rent seekers, scammers, and sociopaths may be possible.<br /><br />
                        </p>
                        <AwesomeButton ripple={true} onPress={() => handleStateChange({readWTF: true})}>
                            join us
                        </AwesomeButton>
                    </>
                ) || (
                    <>
                        <h1 id="mintTitle">
                            WTF
                        </h1>
                        <p id="mintText">
                            This is a community-focused project with the first round being a chain based game to determine the entrants. There is a max supply of {options.unaboomerMaxSupply} Unaboomer tokens (BOOMR) and an infinite supply of Mailbomb tokens (BOMB).
                            Players will mint BOOMR and send BOMB to kill competing players. There is an element of randomness which may cause your bomb to be a dud, or worse, an accidental explosion, killing your token in the process.<br /><br />

                            This is the first round of the game in which we select the entrants to lead the decentralized revolution.
                            There will be no more than {options.unaboomerMaxSurvivorCount} survivors. The round will conclude when the survivor count has been reached.<br /><br />
                        </p>
                        <p>
                            Godspeed.
                        </p>
                        <span id="team">
                            <span className="teamMember">
                                <img src={LZA} width="150px" />
                                <p className="nopad"><a href="https://twitter.com/lza_menace" target="_blank">lza_menace</a> - dev</p>
                            </span>
                            <span className="teamMember">
                                <img src={Carty} width="150px" />
                                <p className="nopad"><a href="https://twitter.com/cartyisme" target="_blank">cartyisme</a> - artist</p>
                            </span>
                        </span>
                        
                        
                        <div className="mintContainer">
                            <div className="mintItem">
                                <h1>Radicalize a Boomer</h1>
                                <p>Radicalizing a boomer will mint ERC-721 BOOMR tokens with images of a pixel art Web3 Unaboomers.</p>
                                <p>You have minted {options.unaboomersMinted} / {options.unaboomerMaxMintPerWallet} BOOMR. {options.unaboomersMinted - options.unaboomerBalance} of your BOOMR were killed. {options.unaboomerBalance} BOOMR left.</p>
                                <img src={Boomer} alt="" width="120px" />
                                {options.unaboomersMinted < options.unaboomerMaxMintPerWallet && options.unaboomersKilled + options.unaboomersRadicalized < options.unaboomerMaxSupply && (
                                    <>
                                        <p>
                                            <AwesomeButton type="secondary" ripple={true} disabled={radicalizeBoomersPrepare.status == 'error'} onPress={() => radicalizeBoomersWrite.write?.()}>
                                                {radicalizeBoomersWrite.isLoading && <>minting {options.unaboomerAmount}</>} 
                                                {radicalizeBoomersWrite.isIdle && <>mint {options.unaboomerPreviewAmount} ({ethers.utils.formatEther((options.unaboomerPrice * options.unaboomerPreviewAmount).toString())} Ξ)</>}
                                                {radicalizeBoomersWrite.isSuccess && <>sent</>}
                                                {radicalizeBoomersWrite.isError && <>error</>}
                                            </AwesomeButton>
                                        </p>
                                        <Slider className="slider" min={1} max={options.unaboomerMaxMintPerWallet - options.unaboomersMinted} onAfterChange={(v) => handleStateChange({unaboomerAmount: v})} onChange={(v) => {
                                            handleStateChange({unaboomerPreviewAmount: v});
                                            radicalizeBoomersWrite.reset();
                                        }} />
                                    </>
                                ) || options.unaboomersMinted == options.unaboomerMaxMintPerWallet && options.unaboomersKilled + options.unaboomersRadicalized < options.unaboomerMaxSupply && (
                                    <h2>max per wallet reached</h2>
                                ) || (
                                    <h2>max supply reached</h2>
                                )}
                            </div>
                            <div className="mintItem">
                                <h1>Assemble Bombs</h1>
                                <p>Assembling bombs will mint ERC-1155 BOMB tokens with an image of a pixel art bomb to send to random Unaboomers to kill them.</p>
                                <p>You have {options.bombBalance} BOMB</p>
                                <img src={Bomb} alt="" width="100px" />
                                <p>
                                    <AwesomeButton type="secondary" ripple={true} disabled={assembleBombsPrepare.status == 'error'} onPress={() => assembleBombsWrite.write?.()}>
                                        {assembleBombsWrite.isLoading && <>minting {options.bombAmount}</>}
                                        {assembleBombsWrite.isIdle && <>mint {options.bombPreviewAmount} ({ethers.utils.formatEther((options.bombPrice * options.bombPreviewAmount).toString())} Ξ)</>}
                                        {assembleBombsWrite.isSuccess && <>sent</>}
                                        {assembleBombsWrite.isError && <>error</>}
                                    </AwesomeButton>
                                </p>
                                <Slider className="slider" min={1} max={50} onAfterChange={(v) => handleStateChange({bombAmount: v})} onChange={(v) => {
                                    handleStateChange({bombPreviewAmount: v});
                                    assembleBombsWrite.reset();
                                }} />
                            </div>
                            <div className="mintItem">
                                <h1>Send Bombs</h1>
                                <p>Sending bombs will burn your BOMB token and kill a BOOMR token at random. Try not to kill yourself in the process.</p>
                                <p>There are {options.unaboomersRadicalized - options.unaboomersKilled} BOOMR available to kill</p>
                                <img src={Explosion} alt="" width="120px" />
                                {options.bombBalance > 0 && (<>
                                    <p>
                                        <AwesomeButton type="secondary" ripple={true} disabled={sendBombsPrepare.status == 'error'} onPress={() => sendBombsWrite.write?.()}>
                                            {sendBombsWrite.isLoading && <>sending {options.sendBombAmount}</>}
                                            {sendBombsWrite.isIdle && <>send {options.sendBombPreviewAmount} (~0 Ξ)</>}
                                            {sendBombsWrite.isSuccess && <>sent</>}
                                            {sendBombsWrite.isError && <>error</>}
                                        </AwesomeButton>
                                    </p>
                                    <Slider className="slider" min={1} max={options.bombBalance} onAfterChange={(v) => handleStateChange({sendBombAmount: v})} onChange={(v) => {
                                        handleStateChange({sendBombPreviewAmount: v});
                                        sendBombsWrite.reset();
                                    }} />
                                </>) || <h2>no bombs</h2>}
                            </div>
                        </div>

                        <div id="userInventory">
                            {isConnected && JSON.parse(tokensMinted).length > 0 && (
                                <>
                                    <h1>Your Unaboomers</h1>
                                    <div className="ownedTokens">
                                        {JSON.parse(tokensMinted).map((id, idx) => (
                                            <div className="ownedToken" key={idx}>
                                                <img src={Boomer} width="80px" />
                                            </div>
                                        ))}
                                    </div>
                                </>
                            )}
                        </div>
                    </>
                )}


                
                <NotificationContainer />
            </>}
            {isConnected && options.unaboomerMaxSupply > 0 && (
                <>
                    <GameMap 
                        bombsAssembled={options.bombsAssembled} 
                        bombBalance={options.bombBalance} 
                        bombsExploded={options.bombsExploded} 
                        unaboomersRadicalized={options.unaboomersRadicalized}
                        unaboomersKilled={options.unaboomersKilled}
                        leaderAddress={options.leaderAddress}
                        leaderKillCount={options.leaderKillCount}
                        unaboomerMaxSupply={options.unaboomerMaxSupply}
                        unaboomerMaxSurvivorCount={options.unaboomerMaxSurvivorCount}
                    />
                    <Trollbox ws={props.ws} address={address} />
                </>
            ) || isConnected && (
                <div id="heroText">
                    <h1>loading...</h1>
                </div>
            )}
            
        </section>
    )
}

function Trollbox(props) {
    const [showChat, setShowChat] = useState(false);
    const [history, setHistory] = useState(() => {
        const saved = localStorage.getItem('trollboxHistory');
        return saved || JSON.stringify([])
    });

    const { sendJsonMessage, readyState } = useWebSocket(props.ws, {
        onOpen: () => {
            console.log('WebSocket connection established.');
        },
        onMessage: (msg) => {
            let h = JSON.parse(history);
            if (h.length > 500) {
                h = h.splice(-500);
            }
            h.push(JSON.parse(msg.data));
            setHistory(JSON.stringify(h));
        },
        shouldReconnect: () => true
    });


    useEffect(() => {
        localStorage.setItem('trollboxHistory', history);
    }, [history]);

    const connectionStatus = {
        [ReadyState.CONNECTING]: 'Connecting',
        [ReadyState.OPEN]: 'Open',
        [ReadyState.CLOSING]: 'Closing',
        [ReadyState.CLOSED]: 'Closed',
        [ReadyState.UNINSTANTIATED]: 'Uninstantiated',
    }[readyState];

    function send(event) {
        const message = (new FormData(event.target)).get("message");
        if (message) {
            sendJsonMessage({
                from: props.address,
                date: new Date(),
                message
            })
        }
        event.target.reset();
        return false;
    }

    function shorten(s) {
        return s.slice(-6)
    }
    
    return (
        <>
            <div id="trollbox">
                {readyState == ReadyState.OPEN && showChat && (
                    <>
                        <button className="button" onClick={() => {
                            setShowChat(false);
                        }}>hide chat</button>
                        <form id="sendForm" onSubmit={(ev) => {
                            ev.preventDefault();
                            send(ev);
                        }}>
                            <input type="text" name="message" minLength="1" />
                            <button type="submit">Send</button>
                        </form>
                        <ul className="nopad">
                            {JSON.parse(history).length > 0 && JSON.parse(history).map((message, idx) => (
                                <li key={idx} className="messageLine">
                                    <span className="fromAddress nopad">
                                        <a href={"https://etherscan.io/address/" + message.from} target="_blank">
                                            {shorten(message.from)}
                                        </a>
                                    :</span>
                                    <span className="messageText">{message.message}</span>
                                    <span className="messageDate">
                                        {new Date(message.date).getMonth() + 1}/
                                        {new Date(message.date).getDate()} @  
                                        {new Date(message.date).getUTCHours()}:
                                        {new Date(message.date).getUTCMinutes()} UTC
                                    </span>
                                </li>
                            ))}
                        </ul>
                        
                    </>
                )}

                {readyState == ReadyState.OPEN && !showChat && (
                    <button className="button" onClick={() => {
                        setShowChat(true);
                    }}>show chat</button>
                )}

                {readyState == ReadyState.CONNECTING && (
                    <p>connecting to trollbox...</p>
                )}
            </div>
        </>
    )
}

export function AllSections(props) {
    return (
        <>
            <Section1 />
            <Section2 />
            <Section3 contractAddress={props.contractAddress} ws={props.ws} />
        </>
    )
}